// Copyright: Hiroshi Ichikawa
// Lincense: New BSD Lincense

#include <string>
#include <vector>
#include "tx.hpp"

class Builder {
    
  public:
    void add(const std::string& word);
    void add_all(const std::vector<std::string>& words);
    void build(const char* file_name);
    
  private:
    
    tx_tool::tx tx_;
    std::vector<std::string> word_list_;
    
};

// This class doesn't perform boundary checking of pos and len.
// lib/tx.rb defines Tx::Index, which is wrapper of this class and performs boundary checking.
// The reason is that we have to call strlen() to know the length of string in C++ code,
// which is O(len of str). Calling str.length in Ruby code is O(1).
class UnsafeIndex {
    
  public:
    
    explicit UnsafeIndex(const char* file_name);
    
    int longest_prefix(const char* str, int pos, int len, bool match_prefix= false);
    bool include(const char* str, int pos, int len);
    std::vector<std::string> search_prefixes(const char* str, int pos, int len);
    std::vector<std::string> search_expansions(const char* str, int pos, int len, int limit= 0);
    
  private:
    
    tx_tool::tx tx_;
    
};
